library ormicida.benchmarks._schemas;

import 'package:ormicida_core/ormicida_core.dart';
import 'package:ormicida_constructors/ormicida_constructors.dart';
import 'package:ormicida_schema/ormicida_schema.dart';

final dynamicPerson = {
  'id': 0,
  'name': 'Tibúrio Bombulá',
  'age': 45,
  'socialNumber': 24245233,
  'email': 'tiburcio@server.dom',
  'address1': {
    'street': 'Overthere 463',
    'code': '2345-678',
    'city': 'Overtheria'},
  'address2': {
    'street': 'Overthere 463',
    'code': '2345-678',
    'city': 'Overtheria'}
};

final mutablePerson = new MutPerson(
  id: 0,
  name: 'Tibúrio Bombulá',
  age: 45,
  socialNumber: 24245233,
  email: 'tiburcio@server.dom',
  address1: new MutAddress(
    street: 'Overthere 463',
    code: '2345-678',
    city: 'Overtheria'),
  address2: new MutAddress(
    street: 'Overthere 463',
    code: '2345-678',
    city: 'Overtheria')
);

final constantPerson = const ConstPerson(
  id: 0,
  name: 'Tibúrio Bombulá',
  age: 45,
  socialNumber: 24245233,
  email: 'tiburcio@server.dom',
  address1: const ConstAddress(
    street: 'Overthere 463',
    code: '2345-678',
    city: 'Overtheria'),
  address2: const ConstAddress(
    street: 'Overthere 463',
    code: '2345-678',
    city: 'Overtheria'));

const ADDRESS_PROPERTIES_STRING = const <Property>[
  const Property('street', const IsString()),
  const Property('code', const IsString()),
  const Property('city', const IsString()),
];
const ADDRESS_PROPERTIES_SYMBOL = const <Property>[
  const Property('street', const IsString(), symbol: const Symbol('street')),
  const Property('code', const IsString(), symbol: const Symbol('code')),
  const Property('city', const IsString(), symbol: const Symbol('city')),
];
const ADDRESS_SCHEMA_MUT_STRING =
  const IsObject(ADDRESS_PROPERTIES_STRING,
      constructor: const DefaultConstructor(MutAddress));
const ADDRESS_SCHEMA_MUT_SYMBOL =
  const IsObject(ADDRESS_PROPERTIES_SYMBOL,
      constructor: const DefaultConstructor(MutAddress));
final ADDRESS_SCHEMA_CONST_STRING =
  new IsObject(ADDRESS_PROPERTIES_STRING, isConst: true,
      constructor: new CustomConstructor(ConstAddress,
          named: {#street: 'street', #code: 'code', #city: 'city'}));
final ADDRESS_SCHEMA_CONST_SYMBOL =
  new IsObject(ADDRESS_PROPERTIES_SYMBOL, isConst: true,
      constructor: new CustomConstructor(ConstAddress,
          named: {#street: 'street', #code: 'code', #city: 'city'}));
final ADDRESS_SCHEMA_CONST_FROM_JSON =
  new IsObject(ADDRESS_PROPERTIES_SYMBOL, isConst: true,
      constructor: (params) => new ConstAddress.fromJson(params));

const PERSON_SCHEMA_MUT_STRING = const IsObject(const <Property>[
  const Property('id', const IsInt(min: 0)),
  const Property('name', const IsString()),
  const Property('age', const IsInt()),
  const Property('socialNumber', const IsInt()),
  const Property('email', const IsString(), symbol: const Symbol('email')),
  const Property('address1', ADDRESS_SCHEMA_MUT_STRING),
  const Property('address2', ADDRESS_SCHEMA_MUT_STRING),
], constructor: const DefaultConstructor(MutPerson));
const PERSON_SCHEMA_MUT_SYMBOL = const IsObject(const <Property>[
  const Property('id', const IsInt(), symbol: const Symbol('id')),
  const Property('name', const IsString(), symbol: const Symbol('name')),
  const Property('age', const IsInt(), symbol: const Symbol('age')),
  const Property('socialNumber', const IsInt(), symbol: const Symbol('socialNumber')),
  const Property('email', const IsString(), symbol: const Symbol('email')),
  const Property('address1', ADDRESS_SCHEMA_MUT_SYMBOL, symbol: const Symbol('address1')),
  const Property('address2', ADDRESS_SCHEMA_MUT_SYMBOL, symbol: const Symbol('address2')),
], constructor: const DefaultConstructor(MutPerson));
final PERSON_SCHEMA_CONST_STRING = new IsObject(<Property>[
  const Property('id', const IsInt()),
  const Property('name', const IsString()),
  const Property('age', const IsInt()),
  const Property('socialNumber', const IsInt()),
  const Property('email', const IsString()),
  new Property('address1', ADDRESS_SCHEMA_CONST_STRING),
  new Property('address2', ADDRESS_SCHEMA_CONST_STRING),
], isConst: true, constructor: new CustomConstructor(ConstPerson, named: {
  #id: 'id',
  #name: 'name',
  #age: 'age',
  #socialNumber: 'socialNumber',
  #email: 'email',
  #address1: 'address1',
  #address2: 'address2'
}));
final PERSON_SCHEMA_CONST_SYMBOL = new IsObject(<Property>[
  const Property('id', const IsInt(), symbol: const Symbol('id')),
  const Property('name', const IsString(), symbol: const Symbol('name')),
  const Property('age', const IsInt(), symbol: const Symbol('age')),
  const Property('socialNumber', const IsInt(), symbol: const Symbol('socialNumber')),
  const Property('email', const IsString(), symbol: const Symbol('email')),
  new Property('address1', ADDRESS_SCHEMA_CONST_SYMBOL, symbol: const Symbol('address1')),
  new Property('address2', ADDRESS_SCHEMA_CONST_SYMBOL, symbol: const Symbol('address2')),
], isConst: true, constructor: new CustomConstructor(ConstPerson, named: {
  #id: 'id',
  #name: 'name',
  #age: 'age',
  #socialNumber: 'socialNumber',
  #email: 'email',
  #address1: 'address1',
  #address2: 'address2'
}));
final PERSON_SCHEMA_CONST_FROM_JSON = new IsObject(<Property>[
  const Property('id', const IsInt(), symbol: const Symbol('id')),
  const Property('name', const IsString(), symbol: const Symbol('name')),
  const Property('age', const IsInt(), symbol: const Symbol('age')),
  const Property('socialNumber', const IsInt(), symbol: const Symbol('socialNumber')),
  const Property('email', const IsString(), symbol: const Symbol('email')),
  new Property('address1', ADDRESS_SCHEMA_CONST_FROM_JSON, symbol: const Symbol('address1')),
  new Property('address2', ADDRESS_SCHEMA_CONST_FROM_JSON, symbol: const Symbol('address2')),
], isConst: true, constructor: (Map params) => new ConstPerson.fromJson(params));

class MutAddress implements Inspectable {
  String street;
  String code;
  String city;
  
  MutAddress({this.street, this.code, this.city});
  
  MutAddress.fromJson(Map json):
    street = json['street'],
    code = json['code'],
    city = json['city'];
  
  void inspect(InspectableSink sink) {
    sink.addProperty('street', street);
    sink.addProperty('code', code);
    sink.addProperty('city', city);
  }
  
  Map toJson() => {'street': street, 'code': code, 'city': city};
  
  operator ==(other) =>
      other is ConstAddress &&
      street == other.street &&
      code == other.code &&
      city == other.city;
  
  String toString() => 'MutAddress(street="$street", code="$code", city="$city")';
}

class MutPerson implements Inspectable {
  int         id;
  String      name;
  int         age;
  int         socialNumber;
  String      email;
  MutAddress  address1;
  MutAddress  address2;
  
  MutPerson({
    this.id,
    this.name,
    this.age,
    this.socialNumber,
    this.email,
    this.address1,
    this.address2
  });
  
  MutPerson.fromJson(Map json):
    id = json['id'],
    name = json['name'],
    age = json['age'],
    socialNumber = json['socialNumber'],
    email = json['email'],
    address1 = json['address1'] != null ? new MutAddress.fromJson(json['address1']) : null,
    address2 = json['address2'] != null ? new MutAddress.fromJson(json['address2']) : null;
  
  void inspect(InspectableSink sink) {
    sink.addProperty('id', id);
    sink.addProperty('name', name);
    sink.addProperty('age', age);
    sink.addProperty('socialNumber', socialNumber);
    sink.addProperty('email', email);
    sink.addProperty('address1', address1);
    sink.addProperty('address2', address2);
  }
    
  Map toJson() => {
    'id': id,
    'name': name,
    'age': age,
    'socialNumber': socialNumber,
    'email': email,
    'address1': address1,
    'address2': address2
  };
  
  operator ==(other) =>
      other is MutPerson &&
      id == other.id &&
      name == other.name &&
      age == other.age &&
      socialNumber == other.socialNumber &&
      email == other.email &&
      address1 == other.address1 &&
      address2 == other.address2;
  
  String toString() => 'MutPerson(id=$id, name="$name", age=$age, socialNumber=$socialNumber, email="$email", address1=$address1, address2:$address2)';
}

class ConstAddress implements Inspectable {
  final String street;
  final String code;
  final String city;
  
  const ConstAddress({this.street, this.code, this.city});
  
  factory ConstAddress.fromJson(Map json) =>
    new ConstAddress(
        street: json['street'],
        code: json['code'],
        city: json['city']);
  
  void inspect(InspectableSink sink) {
    sink.addProperty('street', street);
    sink.addProperty('code', code);
    sink.addProperty('city', city);
  }
  
  Map toJson() => {'street': street, 'code': code, 'city': city};
  
  operator ==(other) =>
      other is ConstAddress &&
      street == other.street &&
      code == other.code &&
      city == other.city;
  
  String toString() => 'ConstAddress(street="$street", code="$code", city="$city")';
}

class ConstPerson implements Inspectable {
  final int           id;
  final String        name;
  final int           age;
  final int           socialNumber;
  final String        email;
  final ConstAddress  address1;
  final ConstAddress  address2;
  
  const ConstPerson({
    this.id,
    this.name,
    this.age,
    this.socialNumber,
    this.email,
    this.address1,
    this.address2
  });
  
  factory ConstPerson.fromJson(Map json) {
    final arg1 = json['address1'];
    final arg2 = json['address2'];
    return new ConstPerson(
      id: json['id'],
      name: json['name'],
      age: json['age'],
      socialNumber: json['socialNumber'],
      email: json['email'],
      address1: arg1 is ConstAddress
                  ? arg1
                  : arg1 is Map
                      ? new ConstAddress.fromJson(arg1)
                      : null,
      address2: arg2 is ConstAddress
                  ? arg2
                  : arg2 is Map
                      ? new ConstAddress.fromJson(arg2)
                      : null);
  }
  
  void inspect(InspectableSink sink) {
    sink.addProperty('id', id);
    sink.addProperty('name', name);
    sink.addProperty('age', age);
    sink.addProperty('socialNumber', socialNumber);
    sink.addProperty('email', email);
    sink.addProperty('address1', address1);
    sink.addProperty('address2', address2);
  }
  
  Map toJson() => {
    'id': id,
    'name': name,
    'age': age,
    'socialNumber': socialNumber,
    'email': email,
    'address1': address1,
    'address2': address2
  };
  
  operator ==(other) =>
      other is ConstPerson &&
      id == other.id &&
      name == other.name &&
      age == other.age &&
      socialNumber == other.socialNumber &&
      email == other.email &&
      address1 == other.address1 &&
      address2 == other.address2;
  
  String toString() => 'ConstPerson(id=$id, name="$name", age=$age, socialNumber=$socialNumber, email="$email", address1=$address1, address2:$address2)';
}
